@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import os
 import sys
+import sysconfig
 from contextlib import contextmanager, nullcontext, suppress
 from functools import cached_property
 from os.path import isabs
@@ -15,11 +16,13 @@ from hatch.utils.shells import ShellManager
 from hatch.utils.structures import EnvVars
 from hatch.venv.core import UVVirtualEnv, VirtualEnv
 
+FREETHREADED_BUILD = bool(sysconfig.get_config_var("Py_GIL_DISABLED"))
+
 if TYPE_CHECKING:
     from collections.abc import Callable, Iterable
 
     from packaging.specifiers import SpecifierSet
-    from virtualenv.discovery.py_info import PythonInfo
+    from python_discovery import PythonInfo
 
     from hatch.dep.core import Dependency
     from hatch.dep.sync import InstalledDistributions
@@ -118,7 +121,7 @@ class VirtualEnvironment(EnvironmentInterface):
             or is_default_environment(env_name, self.app.project.config.internal_envs[env_name])
         ):
             uv_env = self.app.project.get_environment(env_name)
-            self.app.project.prepare_environment(uv_env)
+            self.app.project.prepare_environment(uv_env, keep_env=bool(os.environ.get(AppEnvVars.KEEP_ENV)))
             with uv_env:
                 return self.platform.modules.shutil.which("uv")
 
@@ -204,11 +207,14 @@ class VirtualEnvironment(EnvironmentInterface):
 
             all_install_args = []
 
-            workspace_deps = [str(dep.path) for dep in self.local_dependencies_complex if dep.path]
+            workspace_deps = [dep for dep in self.local_dependencies_complex if dep.path]
             workspace_names = {dep.name.lower() for dep in self.local_dependencies_complex if dep.path}
 
-            for dep_path in workspace_deps:
-                all_install_args.extend(["--editable", dep_path])
+            for dep in workspace_deps:
+                if dep.editable:
+                    all_install_args.extend(["--editable", dep.path])
+                else:
+                    all_install_args.append(dep.path)
 
             standard_dependencies = []
 
@@ -279,7 +285,12 @@ class VirtualEnvironment(EnvironmentInterface):
 
     @cached_property
     def _preferred_python_version(self):
-        return f"{sys.version_info.major}.{sys.version_info.minor}"
+        version = f"{sys.version_info.major}.{sys.version_info.minor}"
+
+        if FREETHREADED_BUILD:
+            version += "t"
+
+        return version
 
     @cached_property
     def parent_python(self):
@@ -323,7 +334,7 @@ class VirtualEnvironment(EnvironmentInterface):
 
     def _interpreter_is_compatible(self, interpreter: PythonInfo) -> bool:
         return (
-            interpreter.executable
+            interpreter.executable is not None
             and self._is_stable_path(interpreter.executable)
             and (self.skip_install or self._python_constraint.contains(interpreter.version_str))
         )
@@ -365,26 +376,12 @@ class VirtualEnvironment(EnvironmentInterface):
         return None
 
     def _find_existing_interpreter(self, python_version: str = "") -> str | None:
-        from virtualenv.discovery import builtin as virtualenv_discovery
+        import python_discovery
 
-        propose_interpreters = virtualenv_discovery.propose_interpreters
-
-        def _patched_propose_interpreters(*args, **kwargs):
-            for interpreter, impl_must_match in propose_interpreters(*args, **kwargs):
-                if not self._interpreter_is_compatible(interpreter):
-                    continue
-
-                yield interpreter, impl_must_match
-
-        virtualenv_discovery.propose_interpreters = _patched_propose_interpreters
-        try:
-            python_info = virtualenv_discovery.get_interpreter(
-                python_version, (), env=self.get_interpreter_resolver_env()
-            )
-            if python_info is not None:
-                return python_info.executable
-        finally:
-            virtualenv_discovery.propose_interpreters = propose_interpreters
+        python_info = python_discovery.get_interpreter(
+            python_version, (), env=self.get_interpreter_resolver_env(), predicate=self._interpreter_is_compatible
+        )
+        return None if python_info is None else python_info.executable
 
     def _get_available_distribution(self, python_version: str = "") -> str | None:
         from hatch.python.resolve import get_compatible_distributions
